@@ -3,7 +3,7 @@ namespace FSpot {
 	public delegate void AreaUpdatedHandler (object sender, Gdk.Rectangle area);
 	public delegate void AreaPreparedHandler (object sender, System.EventArgs args);
 	
-	public class AsyncPixbufLoader {
+	public class AsyncPixbufLoader : System.IDisposable {
 		System.IO.Stream stream;
 		Gdk.PixbufLoader loader;		
 		string path;
@@ -11,9 +11,10 @@ namespace FSpot {
 		bool done_reading = false;
 		System.Exception error;
 		Gdk.Pixbuf pixbuf;
-		Gdk.Pixbuf old;
 		PixbufOrientation orientation;
 
+
+
 		//byte [] buffer = new byte [8192];
 		byte [] buffer = new byte [32768];
 
@@ -21,8 +22,18 @@ namespace FSpot {
 		public event AreaPreparedHandler AreaPrepared;
 		public event System.EventHandler Done;
 
+		// If the photo we just loaded has an out of date
+		// thumbnail save a new one
+		bool validate_thumbnail = true;
+
+		// Limit pass control back to the main loop after
+		// chunk_timeout miliseconds.
+		int  chunk_timeout = 500;
+
 		Delay delay;
 
+		Gdk.Rectangle damage;
+
 		public AsyncPixbufLoader ()
 		{
 			delay = new Delay (new GLib.IdleHandler (AsyncRead));
@@ -40,16 +51,19 @@ namespace FSpot {
 			delay.Stop ();
 			path = filename;
 
-			if (!done_reading && loader != null)
+			if (!done_reading)
 				Close ();
 
 			done_reading = false;
 			area_prepared = false;
+			damage = Gdk.Rectangle.Zero;
 
-			if (stream != null)
-				stream.Close ();
-			
-			orientation = PixbufUtils.GetOrientation (filename);
+			try {
+				orientation = PixbufUtils.GetOrientation (filename);
+			} catch (System.Exception e) {
+				System.Console.WriteLine (e.ToString ());
+				orientation = PixbufOrientation.TopLeft;
+			}
 
 			stream = new System.IO.FileStream (filename, System.IO.FileMode.Open, System.IO.FileAccess.Read);
 			
@@ -71,28 +85,54 @@ namespace FSpot {
 	        public void LoadToAreaPrepared ()
 		{
 			delay.Stop  ();
-			while (AsyncRead () && !area_prepared)
+			while (!area_prepared && AsyncRead ())
 				; //step
 		}
 
 		public void LoadToDone ()
 		{
 			delay.Stop ();
-			while (AsyncRead ())
+			while (!done_reading && AsyncRead ())
 				; //step
 		}
 
 		private void Close () 
 		{
 			try {
-				loader.Close ();
+				delay.Stop ();
+				if (loader != null) {
+					loader.Close ();
+					loader.Dispose ();
+				}
+
+				loader = null;
 			} catch (System.Exception e) {
+				if (pixbuf != null)
+					pixbuf.Dispose ();
+
 				pixbuf = null;
 			} finally {
-				stream.Close ();
+				if (stream != null) 
+					stream.Close ();
+				
+				stream = null;
 			}
 		}
 
+		private void UpdateListeners ()
+		{
+			Gdk.Rectangle area = damage;
+			
+			if (pixbuf != null && loader.Pixbuf != pixbuf)
+				area = PixbufUtils.TransformAndCopy (loader.Pixbuf, pixbuf, orientation, damage);
+			
+			if (AreaUpdated != null)
+				AreaUpdated (this, area);
+
+			damage = Gdk.Rectangle.Zero;
+			System.Console.WriteLine (area.ToString ());
+		}
+
 		private bool AsyncRead () 
 		{
 			System.DateTime start_time = System.DateTime.Now;
@@ -105,21 +145,29 @@ namespace FSpot {
 				loader.Write (buffer, (uint)len);
 				
 				if (len <= 0) {
+					UpdateListeners ();
 					done_reading = true;
 					Close ();
 					return false;
 				}
-			} while (!done_reading && span.TotalMilliseconds <= 300);
+			} while (!done_reading && span.TotalMilliseconds <= 1000);
+
+			UpdateListeners ();
 			return true;
 		}
 		
 		private void HandleAreaPrepared (object sender, System.EventArgs args)
 		{
-			pixbuf = PixbufUtils.TransformOrientation (loader.Pixbuf, orientation);
+			Gdk.Pixbuf old = pixbuf;
+			pixbuf = PixbufUtils.TransformOrientation (loader.Pixbuf, orientation, false);
 
 			area_prepared = true;			
 			if (AreaUpdated != null)
 				AreaPrepared (this, System.EventArgs.Empty);
+
+			if (old != null) {
+				old.Dispose ();
+			}
 		}
 
 		public Gdk.Pixbuf Pixbuf {
@@ -134,23 +182,37 @@ namespace FSpot {
 		private void HandleAreaUpdated (object sender, Gdk.AreaUpdatedArgs args)
 		{
 			Gdk.Rectangle area = new Gdk.Rectangle (args.X, args.Y, args.Width, args.Height);
-
-			if (pixbuf != null && loader.Pixbuf != pixbuf)
-				area = PixbufUtils.TransformAndCopy (loader.Pixbuf, pixbuf, orientation, area);
-
-			if (AreaUpdated != null)
-				AreaUpdated (this, area);
+			
+			if (damage.Width == 0 || damage.Height == 0)
+				damage = area;
+			else 
+				damage = area.Union (damage);
 		}
 
 		private void HandleClosed (object sender, System.EventArgs args) 
 		{
-			if (done_reading && pixbuf != null) {
+			// FIXME This should probably queue the
+			// thumbnail regeneration to a worker thread
+			if (validate_thumbnail && done_reading && pixbuf != null) {
 				PhotoLoader.ValidateThumbnail (path, pixbuf);
 			}
 
 			if (Done != null)
 				Done (this, System.EventArgs.Empty);
 		}
+
+		public void Dispose ()
+		{
+			if (loader != null) {
+				loader.AreaPrepared -= HandleAreaPrepared;
+				loader.AreaUpdated -= HandleAreaUpdated;
+				loader.Closed -= HandleClosed;
+			}
+			Close ();
+			
+			if (pixbuf != null)
+				pixbuf.Dispose ();
+		}
 	}
 }
 
