@@ -6,13 +6,12 @@ using Gtk;
 using Gnome;
 
 public class DateCommands {
-
 	public class Set {
-		PhotoQuery query;
+		FSpot.PhotoQuery query;
 		Gtk.Window parent_window;
 
 		[Glade.Widget]
-		private Dialog date_range_dialog;
+		private Gtk.Dialog date_range_dialog;
 
 		[Glade.Widget]
 		private Button ok_button;
@@ -20,9 +19,15 @@ public class DateCommands {
 		[Glade.Widget]
 		private DateEdit start_dateedit;
 
-		[Glade.Widget]
+		[Glade.Widget] 
 		private DateEdit end_dateedit;
 
+		public Set (FSpot.PhotoQuery query, Gtk.Window parent_window)
+		{
+			this.query = query;
+			this.parent_window = parent_window;
+		}
+
 		public bool Execute ()
 		{
 			Glade.XML xml = new Glade.XML (null, "f-spot.glade", "date_range_dialog", null);
@@ -45,13 +50,6 @@ public class DateCommands {
 			
 			date_range_dialog.Destroy ();
 			return success;
-
-		}
-
-		public Set (PhotoQuery query, Gtk.Window parent_window)
-		{
-			this.query = query;
-			this.parent_window = parent_window;
 		}
 	}
 }
