@@ -2,7 +2,6 @@ using Gdk;
 using Gtk;
 using Gnome;
 using System.Collections;
-using System.IO;
 using System;
 
 public class FileImportBackend : ImportBackend {
@@ -21,19 +20,36 @@ public class FileImportBackend : ImportBackend {
 			file_paths.Add (path);
 
 	}
-
-	private void GetListing (DirectoryInfo info)
+	
+	private void GetListing (System.IO.DirectoryInfo info)
 	{
-		FileInfo [] files = info.GetFiles ();
+		System.Console.WriteLine ("Sanning {0}", info.FullName);
 
-		foreach (FileInfo f in files) {
-			string path = f.FullName;
-			
-			AddPath (path);
+		System.IO.FileInfo [] files = info.GetFiles ();
+		Hashtable exiting_entries = new Hashtable ();
+
+		foreach (Photo p in store.Query (info)) {
+			foreach (uint id in p.VersionIds) {
+				string name;
+				if (id == Photo.OriginalVersionId)
+				        name = p.Name;
+				else 
+					name = (new System.IO.FileInfo (p.GetVersionPath (id))).Name;
+
+				System.Console.WriteLine ("Found {0}", name);
+				exiting_entries [name] = p;
+			}
+		}
+	
+		foreach (System.IO.FileInfo f in files) {
+			if (exiting_entries [f.Name] == null) {
+				System.Console.WriteLine ("Importing {0} as {1}", f.Name, f.FullName);
+				AddPath (f.FullName);
+			}
 		}
 
 		if (recurse) {
-			foreach (DirectoryInfo d in info.GetDirectories ()){
+			foreach (System.IO.DirectoryInfo d in info.GetDirectories ()){
 				if (!d.Name.StartsWith ("."))
 					GetListing (d);
 			}
@@ -49,8 +65,9 @@ public class FileImportBackend : ImportBackend {
 
 		foreach (string path in base_paths) {
 			try {	
-				GetListing (new DirectoryInfo (path));
-			} catch {
+				GetListing (new System.IO.DirectoryInfo (path));
+			} catch (Exception e) {
+				System.Console.WriteLine (e.ToString ());
 				AddPath (path);
 			}
 		}	
