@@ -64,6 +64,17 @@ public class IconView : Gtk.Layout {
 		}
 	}
 
+	private bool display_dates = true;
+	public bool DisplayDates {
+		get {
+			return display_dates;
+		}
+
+		set {
+			display_dates = value;
+			QueueResize ();
+		}
+	}
 
 	// Size of the frame around the thumbnail.
 	private const int CELL_BORDER_WIDTH = 10;
@@ -75,7 +86,7 @@ public class IconView : Gtk.Layout {
 	private const int SELECTION_THICKNESS = 5;
 
 	// Size of the tag icon in the view.
-	private const int TAG_ICON_SIZE = 12;
+	private const int TAG_ICON_SIZE = 16;
 
 	// Horizontal spacing between the tag icons
 	private const int TAG_ICON_HSPACING = 2;
@@ -89,12 +100,8 @@ public class IconView : Gtk.Layout {
 	// Various other layout values.
 	private int cells_per_row;
 	private int cell_width;
-
 	private int cell_height;
 
-	// Query we are displaying.
-	private PhotoQuery query;
-
 	// The first pixel line that is currently on the screen (i.e. in the current
 	// scroll region).  Used to compute the area that went offscreen in the "changed"
 	// signal handler for the vertical GtkAdjustment.
@@ -140,18 +147,16 @@ public class IconView : Gtk.Layout {
 
 	public IconView () : base (null, null)
 	{
-		pixbuf_loader = new PixbufLoader (256);
-		pixbuf_loader.OnPixbufLoaded += new PixbufLoader.PixbufLoadedHandler (HandlePixbufLoaded);
+		pixbuf_loader = new PixbufLoader ();
+		pixbuf_loader.OnPixbufLoaded += HandlePixbufLoaded;
 
 		selected_cells = new Hashtable ();
 
 		ScrollAdjustmentsSet += new ScrollAdjustmentsSetHandler (HandleScrollAdjustmentsSet);
-		SizeAllocated += new SizeAllocatedHandler (HandleSizeAllocated);
 		
 		ButtonPressEvent += new ButtonPressEventHandler (HandleButtonPressEvent);
 		ButtonReleaseEvent += new ButtonReleaseEventHandler (HandleButtonReleaseEvent);
 		KeyPressEvent += new KeyPressEventHandler (HandleKeyPressEvent);
-		KeyReleaseEvent += new KeyReleaseEventHandler (HandleKeyReleaseEvent);
 
 		DestroyEvent += new DestroyEventHandler (HandleDestroyEvent);
 
@@ -160,30 +165,26 @@ public class IconView : Gtk.Layout {
 			   | (int) EventMask.PointerMotionMask);
 		
 		CanFocus = true;
-	}
 
-	private void OnReload (PhotoQuery query)
-	{
-		// FIXME we should probably try to merge the selection forward
-		// but it needs some thought to be efficient.
-		UnselectAllCells ();
-		QueueResize ();
+		FSpot.Global.ModifyColors (this);
 	}
-
-	public IconView (PhotoQuery query) : this ()
+	
+	public IconView (FSpot.IBrowsableCollection collection) : this () 
 	{
-		this.query = query;
-		query.Reload += new PhotoQuery.ReloadHandler (OnReload);
+		this.collection = collection;
 	}
+	
 
 	protected IconView (IntPtr raw) : base (raw) {}
 
 	//
 	// IPhotoSelection
 	//
-	public PhotoQuery Query {
+
+	protected FSpot.IBrowsableCollection collection;
+	public FSpot.IBrowsableCollection Collection {
 		get {
-			return query;
+			return collection;
 		}
 	}
 
@@ -222,11 +223,10 @@ public class IconView : Gtk.Layout {
 	}
 
 	// Updating.
-
 	public void UpdateThumbnail (int thumbnail_num)
 	{
-		Photo photo = query.Photos [thumbnail_num];
-		string thumbnail_path = Thumbnail.PathForUri ("file://" + photo.DefaultVersionPath, ThumbnailSize.Large);
+		FSpot.IBrowsableItem photo = collection.Items [thumbnail_num];
+		string thumbnail_path = Thumbnail.PathForUri (photo.DefaultVersionUri.ToString (), ThumbnailSize.Large);
 
 		//Console.WriteLine ("remove {0}", thumbnail_path);
 		ThumbnailCache.Default.RemoveThumbnailForPath (thumbnail_path);
@@ -238,19 +238,19 @@ public class IconView : Gtk.Layout {
 
 	public int CellAtPosition (int x, int y)
 	{
-		if (query == null)
+		if (collection == null)
 			return -1;
 
 		if (x < BORDER_SIZE || x >= BORDER_SIZE + cells_per_row * cell_width)
 			return -1;
-		if (y < BORDER_SIZE || y >= BORDER_SIZE + (query.Photos.Length / cells_per_row + 1) * cell_height)
+		if (y < BORDER_SIZE || y >= BORDER_SIZE + (collection.Items.Length / cells_per_row + 1) * cell_height)
 			return -1;
 
 		int column = (int) ((x - BORDER_SIZE) / cell_width);
 		int row = (int) ((y - BORDER_SIZE) / cell_height);
 		int cell_num = column + row * cells_per_row;
 
-		if (cell_num < query.Photos.Length)
+		if (cell_num < collection.Items.Length)
 			return (int) cell_num;
 		else
 			return -1;
@@ -276,15 +276,6 @@ public class IconView : Gtk.Layout {
 
 
 	// Private utility methods.
-
-	static private Pixbuf ErrorPixbuf ()
-	{
-		if (IconView.error_pixbuf == null)
-			IconView.error_pixbuf = PixbufUtils.LoadFromAssembly ("f-spot-question-mark.png");
-
-		return IconView.error_pixbuf;
-	}
-
 	public void UnselectAllCells ()
 	{
 		if (selected_cells.Count == 0)
@@ -329,7 +320,7 @@ public class IconView : Gtk.Layout {
 
 	public void SelectAllCells ()
 	{
-		SelectCellRange (0, query.Photos.Length - 1);
+		SelectCellRange (0, collection.Items.Length - 1);
 	}
 
 	private void SelectCellRange (int start, int end)
@@ -372,7 +363,15 @@ public class IconView : Gtk.Layout {
 
 
 	// Layout and drawing.
-
+	
+	// FIXME I can't find a c# wrapper for the C PANGO_PIXELS () macro
+	// So this Function is for that.
+	static int PangoPixels (int val)
+	{
+		return val >= 0 ? (val + 1024 / 2) / 1024 :
+			(val - 1024 / 2) / 1024;
+	}
+	
 	private void UpdateLayout ()
 	{
 		int available_width = Allocation.Width - 2 * BORDER_SIZE;
@@ -382,14 +381,18 @@ public class IconView : Gtk.Layout {
 
 		if (DisplayTags)
 			cell_height += TAG_ICON_SIZE + TAG_ICON_VSPACING;
+		
+		if (DisplayDates && this.Style != null) {
+			Pango.FontMetrics metrics = this.PangoContext.GetMetrics (this.Style.FontDescription, 
+										  Pango.Language.FromString ("en_US"));
+			cell_height += PangoPixels (metrics.Ascent + metrics.Descent);
+		}
 
-		cells_per_row = (int) (available_width / cell_width);
-		if (cells_per_row == 0)
-			cells_per_row = 1;
+		cells_per_row = Math.Max ((int) (available_width / cell_width), 1);
 
 		int num_thumbnails;
-		if (query != null)
-			num_thumbnails = query.Photos.Length;
+		if (collection != null)
+			num_thumbnails = collection.Items.Length;
 		else
 			num_thumbnails = 0;
 
@@ -403,82 +406,146 @@ public class IconView : Gtk.Layout {
 		Vadjustment.Change ();
 	}
 
+	System.Collections.Hashtable date_layouts = new Hashtable ();
 	// FIXME Cache the GCs?
-	private void DrawCell (int thumbnail_num, int x, int y)
+	private void DrawCell (int thumbnail_num, int x, int y, Gdk.Rectangle area)
 	{
 		Gdk.GC gc = new Gdk.GC (BinWindow);
 		gc.Copy (Style.ForegroundGC (StateType.Normal));
 		gc.SetLineAttributes (1, LineStyle.Solid, CapStyle.NotLast, JoinStyle.Round);
 		bool selected = CellIsSelected (thumbnail_num);
 
-		Photo photo = query.Photos [thumbnail_num];
+		FSpot.IBrowsableItem photo = collection.Items [thumbnail_num];
 
-		string thumbnail_path = Thumbnail.PathForUri ("file://" + photo.DefaultVersionPath, ThumbnailSize.Large);
+		string thumbnail_path = Thumbnail.PathForUri (photo.DefaultVersionUri.ToString (), ThumbnailSize.Large);
 		Pixbuf thumbnail = ThumbnailCache.Default.GetThumbnailForPath (thumbnail_path);
+			
+		StateType cell_state = selected ? (HasFocus ? StateType.Selected :StateType.Active) : StateType.Normal;
 
-		Gdk.Rectangle area = new Gdk.Rectangle (x, y, cell_width, cell_height);
-		
-#if false
-		Style.PaintFlatBox (Style, BinWindow, selected ? (HasFocus ? StateType.Selected :StateType.Active) : StateType.Normal, 
-				    ShadowType.Out, area, this, null, x, y, cell_width, cell_height);
 
-#else
-		Style.PaintBox (Style, BinWindow, selected ? (HasFocus ? StateType.Selected :StateType.Active) : StateType.Normal, 
-				ShadowType.Out, area, this, "IconView", x, y, cell_width, cell_height);
+		Style.PaintFlatBox (Style, BinWindow, cell_state, 
+				    ShadowType.Out, area, this, "IconView", x, y, cell_width - 1, cell_height - 1);
 
-#endif 
 		if (HasFocus && thumbnail_num == FocusCell) {
-			Style.PaintFocus(Style, BinWindow, StateType.Normal, area, 
+			Style.PaintFocus(Style, BinWindow, cell_state, area, 
 					 this, null, x + 3, y + 3, cell_width - 6, cell_height - 6);
 		}
 
-		if (thumbnail == null) {
-			pixbuf_loader.Request (thumbnail_path, thumbnail_num);
-		} else {
-			int width, height;
-			PixbufUtils.Fit (thumbnail, ThumbnailWidth, ThumbnailHeight, true, out width, out height);
+		Gdk.Rectangle image_area = new Gdk.Rectangle (x + CELL_BORDER_WIDTH, 
+							      y + CELL_BORDER_WIDTH, 
+							      cell_width - 2 * CELL_BORDER_WIDTH, 
+							      cell_height - 2 * CELL_BORDER_WIDTH);
+		Gdk.Rectangle result = Rectangle.Zero;
 
-			int dest_x = (int) (x + (cell_width - width) / 2);
-			int dest_y;
-			if (DisplayTags)
-				dest_y = (int) (y + (cell_height - height - (TAG_ICON_SIZE + TAG_ICON_VSPACING)) / 2);
-			else
-				dest_y = (int) (y + (cell_height - height) / 2);
-			
+		int layout_width = 0;
+		int layout_height = 0;		
+
+		if (image_area.Intersect (area, out result)) {
+			int expansion = 0;
 			if (thumbnail_num == throb_cell) {
 				double t = throb_state / (double) (throb_state_max - 1);
 				double s;
-				if (selected)
-					s = Math.Cos (-2 * Math.PI * t);
-				else
-					s = 1 - Math.Cos (-2 * Math.PI * t);
-
-				int scale = (int) (SELECTION_THICKNESS * s);
-				dest_x -= scale;
-				dest_y -= scale;		
-				width += 2 * scale;
-				height += 2 * scale;
-			} else 	if (selected) { 
-				dest_x -= SELECTION_THICKNESS;
-				dest_y -= SELECTION_THICKNESS;
-				width += 2 * SELECTION_THICKNESS;
-				height += 2 * SELECTION_THICKNESS;
+			if (selected)
+				s = Math.Cos (-2 * Math.PI * t);
+			else
+				s = 1 - Math.Cos (-2 * Math.PI * t);
+			
+			expansion = (int) (SELECTION_THICKNESS * s);
+			} else if (selected) {
+				expansion = SELECTION_THICKNESS;
 			}
+			
+			if (thumbnail == null) {
+				// FIXME instead of making a request to load at a particular size here we
+				// request to load at the the full size because Gdk.Pixbuf loses the Option
+				// data when you load at a different size (I hate that).  See HandlePixbufLoaded
+				// for where we do the scaling to correct for this problem.
+				pixbuf_loader.Request (thumbnail_path, thumbnail_num);
+			}
+			
+			if (thumbnail != null){
+				int width, height;
+				
+				PixbufUtils.Fit (thumbnail, ThumbnailWidth, ThumbnailHeight, true, out width, out height);
+				
+				int dest_x = (int) (x + (cell_width - width) / 2);
+				int dest_y;
+				
+				
+				dest_y = (int) y + ThumbnailHeight - height + CELL_BORDER_WIDTH;
+				
+				dest_x -= expansion;
+				dest_y -= expansion;		
+				width += 2 * expansion;
+				height += 2 * expansion;
 
-			Pixbuf temp_thumbnail;
-			if (width == thumbnail.Width) {
-				temp_thumbnail = thumbnail;
+				Pixbuf temp_thumbnail;
+				// Is the thumbnail the right size?
+				if (width == thumbnail.Width || height == thumbnail.Height) {
+					temp_thumbnail = thumbnail;
+				} else {
+
+					if (width < thumbnail.Width && height < thumbnail.Height)
+						temp_thumbnail = PixbufUtils.ScaleDown (thumbnail, width, height);
+					else
+						temp_thumbnail = thumbnail.ScaleSimple (width, height, InterpType.Bilinear);
+					
+					PixbufUtils.CopyThumbnailOptions (thumbnail, temp_thumbnail);
+					
+					// Only request a reload here if we are not inside a throb 
+					// (expansion == 0) because we want to store the images at the normal
+					// size not a throb or selection size to optimize the scroll speed
+					if (expansion == 0) {
+#if false						
+						System.Console.WriteLine ("made request for {0} ({1},{2}) ({3},{4}) ({5},{6})", 
+									  thumbnail_path, width, height, 
+									  ThumbnailWidth, ThumbnailHeight, 
+									  thumbnail.Width, thumbnail.Height);
+#endif
+
+						// FIXME instead of making a request to load at a particular size here we
+						// request to load at the the full size because Gdk.Pixbuf loses the Option
+						// data when you load at a different size (I hate that).  See HandlePixbufLoaded
+						// for where we do the scaling to correct for this problem.
+						pixbuf_loader.Request (thumbnail_path, thumbnail_num);
+					}
+				}
+				
+				Style.PaintShadow (Style, BinWindow, cell_state,
+						   ShadowType.Out, area, this, "IconView", dest_x - 1, dest_y - 1, width + 2, height + 2);			
+				temp_thumbnail.RenderToDrawable (BinWindow, Style.WhiteGC,
+								 0, 0, dest_x, dest_y, width, height, RgbDither.None, 0, 0);
+				
+				if (temp_thumbnail != thumbnail)
+					temp_thumbnail.Dispose ();
+				
+				thumbnail.Dispose ();
+			}
+		}
+			
+		if (DisplayDates) {
+			string date;
+			if (cell_width > 200) {
+				date = photo.Time.ToString ();
 			} else {
-				temp_thumbnail = thumbnail.ScaleSimple (width, height, InterpType.Bilinear);
+				date = photo.Time.ToShortDateString ();
 			}
 
-			temp_thumbnail.RenderToDrawable (BinWindow, Style.WhiteGC,
-							 0, 0, dest_x, dest_y, width, height, RgbDither.None, 0, 0);
+			Pango.Layout layout = (Pango.Layout)date_layouts [date];
+			if (layout == null) {
+				layout = new Pango.Layout (this.PangoContext);
+				layout.SetText (date);
+				date_layouts [date] = layout;
+			}
 			
-			if (temp_thumbnail != thumbnail)
-				temp_thumbnail.Dispose ();
-			
-			thumbnail.Dispose ();
+			layout.GetPixelSize (out layout_width, out layout_height);
+
+			int layout_y = y + cell_height - CELL_BORDER_WIDTH - (DisplayTags ? TAG_ICON_SIZE : 0) - layout_height;
+			int layout_x = x + (cell_width - layout_width) / 2;
+
+			Style.PaintLayout (Style, BinWindow, cell_state,
+					   true, area, this, "IconView", layout_x, layout_y, layout);
+
 		}
 
 		if (DisplayTags) {
@@ -486,7 +553,7 @@ public class IconView : Gtk.Layout {
 
 			int tag_x, tag_y;
 
-			tag_x = x + CELL_BORDER_WIDTH;
+			tag_x = x + (cell_width - tags.Length * TAG_ICON_SIZE) / 2;
 			tag_y = y + cell_height - CELL_BORDER_WIDTH - TAG_ICON_SIZE;
 			foreach (Tag t in tags) {
 				Pixbuf icon = null;
@@ -513,21 +580,24 @@ public class IconView : Gtk.Layout {
 							      0, 0, tag_x, tag_y, TAG_ICON_SIZE, TAG_ICON_SIZE,
 							      RgbDither.None, 0, 0);
 				tag_x += TAG_ICON_SIZE + TAG_ICON_VSPACING;
+
+				if (scaled_icon != icon)
+					scaled_icon.Dispose ();
 			}
 		}
 	}
 
-	private void DrawAllCells (int x, int y, int width, int height)
+	private void DrawAllCells (Gdk.Rectangle area)
 	{
-		int start_cell_column = Math.Max ((x - BORDER_SIZE) / cell_width, 0);
-		int start_cell_row = Math.Max ((y - BORDER_SIZE) / cell_height, 0);
+		int start_cell_column = Math.Max ((area.X - BORDER_SIZE) / cell_width, 0);
+		int start_cell_row = Math.Max ((area.Y - BORDER_SIZE) / cell_height, 0);
 		int start_cell_num = start_cell_column + start_cell_row * cells_per_row;
 
 		int start_cell_x, cell_y;
 		GetCellPosition (start_cell_num, out start_cell_x, out cell_y);
 
-		int end_cell_column = Math.Max ((x + width - BORDER_SIZE) / cell_width, 0);
-		int end_cell_row = Math.Max ((y + height - BORDER_SIZE) / cell_height, 0);
+		int end_cell_column = Math.Max ((area.X + area.Width - BORDER_SIZE) / cell_width, 0);
+		int end_cell_row = Math.Max ((area.Y + area.Height - BORDER_SIZE) / cell_height, 0);
 
 		int num_rows = end_cell_row - start_cell_row + 1;
 		int num_cols = Math.Min (end_cell_column - start_cell_column + 1,
@@ -535,14 +605,16 @@ public class IconView : Gtk.Layout {
 
 		int i, cell_num;
 		for (i = 0, cell_num = start_cell_num;
-		     i < num_rows && cell_num < query.Photos.Length;
+		     i < num_rows && cell_num < collection.Items.Length;
 		     i ++) {
 			int cell_x = start_cell_x;
 
 			//Console.WriteLine ("Drawing row {0}", start_cell_row + i);
-			for (int j = 0; j < num_cols && cell_num + j < query.Photos.Length; j ++) {
-				//Console.WriteLine ("Drawing Cell {0}", cell_num + j);
-				DrawCell (cell_num + j, cell_x, cell_y);
+			for (int j = 0; j < num_cols && cell_num + j < collection.Items.Length; j ++) {
+				Gdk.Rectangle cell_bounds = CellBounds (cell_num + j);
+				if (area.Intersect (cell_bounds, out cell_bounds)) {
+					DrawCell (cell_num + j, cell_x, cell_y, cell_bounds);
+				}
 				cell_x += cell_width;
 			}
 
@@ -575,7 +647,7 @@ public class IconView : Gtk.Layout {
 		if (y_offset == adjustment.Value)
 			return false;
 
-		int num_thumbnails = query.Photos.Length;
+		int num_thumbnails = collection.Items.Length;
 		int num_rows, start;
 
 		if (y_offset < adjustment.Value) {
@@ -596,8 +668,8 @@ public class IconView : Gtk.Layout {
 			if (start + i >= num_thumbnails)
 				break;
 
-			Photo photo = query.Photos [start + i];
-			string thumbnail_path = Thumbnail.PathForUri ("file://" + photo.DefaultVersionPath, ThumbnailSize.Large);
+			FSpot.IBrowsableItem photo = collection.Items [start + i];
+			string thumbnail_path = Thumbnail.PathForUri (photo.DefaultVersionUri.ToString (), ThumbnailSize.Large);
 			pixbuf_loader.Cancel (thumbnail_path);
 		}
 
@@ -647,7 +719,7 @@ public class IconView : Gtk.Layout {
 	private bool HandleThrobTimer () 
 	{
 		//Console.WriteLine ("throb out {1} {0}", throb_cell, 1 - Math.Cos (throb_state));
-		
+
 		InvalidateCell (throb_cell);
 		if (throb_state++ < throb_state_max) {
 			return true;
@@ -671,7 +743,11 @@ public class IconView : Gtk.Layout {
 		else if (y + cell_height > adjustment.Value + adjustment.PageSize)
 			adjustment.Value = y + cell_height - adjustment.PageSize;
 		
-		adjustment.ChangeValue();
+#if USEIDLE
+		Scroll ();
+#else
+		adjustment.Change ();
+#endif		
 	}
 
 
@@ -685,15 +761,33 @@ public class IconView : Gtk.Layout {
 	private void HandlePixbufLoaded (PixbufLoader loader, string path, int order, Pixbuf result)
 	{
 		if (result == null) {
-			result = ErrorPixbuf ();
+			result = PixbufUtils.ErrorPixbuf;
 			//
 			// ThumbnailCache Takes Ownership and calls Dispose
 			// so we need to copy the ErrorPixbuf
 			//
 			result = new Pixbuf (result, 0, 0, result.Width, result.Height);
+		} 
+
+		if (order > 0 && order < collection.Items.Length) {
+			System.Uri uri = collection.Items [order].DefaultVersionUri;
+			
+			if (!FSpot.PhotoLoader.ThumbnailIsValid (collection.Items [order].DefaultVersionUri, result)) {
+				System.Console.WriteLine ("regnerating thumbnail");
+				FSpot.ThumbnailGenerator.Default.Request (uri.LocalPath, 0, 256, 256);
+			}
+		}
+		       
+
+		// We have to do the scaling here rather than on load because we need to preserve the 
+		// Pixbuf option iformation to verify the thumbnail validity later
+		if (result.Width != ThumbnailWidth && result.Height != ThumbnailHeight) {
+			Gdk.Pixbuf temp = PixbufUtils.ScaleToMaxSize (result, ThumbnailWidth, ThumbnailHeight);
+			PixbufUtils.CopyThumbnailOptions (result, temp);
+			result.Dispose ();
+			result = temp;
 		}
 
-		//Console.WriteLine ("adding {0}", path);
 		ThumbnailCache.Default.AddThumbnail (path, result);
 		InvalidateCell (order);
 	}
@@ -710,6 +804,9 @@ public class IconView : Gtk.Layout {
 	public void InvalidateCell (int order) 
 	{
 		Rectangle cell_area = CellBounds (order);
+		// FIXME where are we computing the bounds incorrectly
+		cell_area.Width -= 1;
+		cell_area.Height -= 1;
 		BinWindow.InvalidateRect (cell_area, false);
 	}
 			
@@ -719,16 +816,17 @@ public class IconView : Gtk.Layout {
 			args.Vadjustment.ValueChanged += new EventHandler (HandleAdjustmentValueChanged);
 	}
 
-	private void HandleSizeAllocated (object sender, SizeAllocatedArgs args)
+	protected override void OnSizeAllocated (Gdk.Rectangle allocation)
 	{
+		base.OnSizeAllocated (allocation);
 		UpdateLayout ();
 	}
 
 	protected override bool OnExposeEvent (Gdk.EventExpose args)
 	{
-		DrawAllCells (args.Area.X, args.Area.Y,
-			      args.Area.Width, args.Area.Height);
-
+		foreach (Rectangle area in args.Region.GetRectangles ()) {
+			DrawAllCells (area);
+		}
 		return base.OnExposeEvent (args);
 	}
 
@@ -845,7 +943,7 @@ public class IconView : Gtk.Layout {
 				FocusCell = 0;
 				break;
 			case Gdk.Key.End:
-				FocusCell = query.Photos.Length - 1; 
+				FocusCell = collection.Items.Length - 1; 
 				break;
 			case Gdk.Key.space:
 				ToggleCell (FocusCell);
@@ -855,7 +953,7 @@ public class IconView : Gtk.Layout {
 				return;		
 		}
 		
-		if (FocusCell < 0 || FocusCell > query.Photos.Length - 1) {
+		if (FocusCell < 0 || FocusCell > collection.Items.Length - 1) {
 			FocusCell = focus_old;
 			args.RetVal = false;
 		}	
@@ -870,11 +968,6 @@ public class IconView : Gtk.Layout {
 		ScrollTo (FocusCell);
 	}
 
-	private void HandleKeyReleaseEvent (object sender, KeyReleaseEventArgs args)
-	{
-		Console.WriteLine ("Release!");
-	}
-
 	private void HandleDestroyEvent (object sender, DestroyEventArgs args)
 	{
 		CancelScroll ();
