@@ -22,8 +22,12 @@ public class MainWindow {
 
 	TagSelectionWidget tag_selection_widget;
 	[Glade.Widget] Gtk.Window main_window;
-	[Glade.Widget] VBox left_vbox;
-	[Glade.Widget] VBox group_vbox;
+	[Glade.Widget] Gtk.VBox left_vbox;
+	[Glade.Widget] Gtk.VBox group_vbox;
+	[Glade.Widget] Gtk.VBox view_vbox;
+
+	[Glade.Widget] Gtk.VBox toolbar_vbox;
+	
 
 	[Glade.Widget] ScrolledWindow icon_view_scrolled;
 	[Glade.Widget] Box photo_box;
@@ -67,17 +71,21 @@ public class MainWindow {
 	FSpot.FullScreenView fsview;
 	FSpot.PhotoQuery query;
 	FSpot.GroupSelector group_selector;
+
+	FSpot.Delay slide_delay;
 	
 	// Drag and Drop
 	enum TargetType {
 		UriList,
 		TagList,
-		PhotoList
+		PhotoList,
+		RootWindow
 	};
 
 	private static TargetEntry [] icon_source_target_table = new TargetEntry [] {
 		new TargetEntry ("application/x-fspot-photos", 0, (uint) TargetType.PhotoList),
 		new TargetEntry ("text/uri-list", 0, (uint) TargetType.UriList),
+		new TargetEntry ("application/x-root-window-drop", 0, (uint) TargetType.RootWindow)
 	};
 
 	private static TargetEntry [] icon_dest_target_table = new TargetEntry [] {
@@ -103,8 +111,10 @@ public class MainWindow {
 
 	// Index into the PhotoQuery.  If -1, no photo is selected or multiple photos are selected.
 	private int ActiveIndex () {
+		if (view_mode == ModeType.IconView && icon_view.CurrentIdx != -1)
+			return icon_view.CurrentIdx;
+
 	        int [] selection = SelectedIds ();
-		
 		if (selection.Length == 1) 
 			return selection [0];
 		else 
@@ -126,13 +136,7 @@ public class MainWindow {
 		}
 	}
 
-	// Switching mode.
-	enum ModeType {
-		IconView,
-		PhotoView
-	};
-	ModeType mode;
-	
+
 	//
 	// Constructor
 	//
@@ -140,10 +144,23 @@ public class MainWindow {
 	{
 		this.db = db;
 
-		Mono.Posix.Catalog.Init ("f-spot", Defines.LOCALE_DIR);
+		Mono.Posix.Catalog.Init ("f-spot", FSpot.Defines.LOCALE_DIR);
 		Glade.XML gui = Glade.XML.FromAssembly ("f-spot.glade", "main_window", null);
 		gui.Autoconnect (this);
 
+		slide_delay = new FSpot.Delay (new GLib.IdleHandler (SlideShow));
+
+		Gtk.Toolbar toolbar = new Gtk.Toolbar ();
+		toolbar_vbox.PackStart (toolbar);
+		GtkUtil.MakeToolbarButton (toolbar, "f-spot-rotate-270", new System.EventHandler (HandleRotate270Command));
+		GtkUtil.MakeToolbarButton (toolbar, "f-spot-rotate-90", new System.EventHandler (HandleRotate90Command));
+		toolbar.AppendSpace ();
+		GtkUtil.MakeToolbarButton (toolbar, "f-spot-browse", new System.EventHandler (HandleViewBrowse));
+		GtkUtil.MakeToolbarButton (toolbar, "f-spot-edit-image", new System.EventHandler (HandleViewPhoto));
+		toolbar.AppendSpace ();
+		GtkUtil.MakeToolbarButton (toolbar, "f-spot-fullscreen", new System.EventHandler (HandleViewFullscreen));
+		GtkUtil.MakeToolbarButton (toolbar, "f-spot-slideshow", new System.EventHandler (HandleViewSlideShow));
+
 		tag_selection_widget = new TagSelectionWidget (db.Tags);
 		tag_selection_scrolled.Add (tag_selection_widget);
 		
@@ -163,28 +180,34 @@ public class MainWindow {
 		info_box = new InfoBox ();
 		info_box.VersionIdChanged += new InfoBox.VersionIdChangedHandler (HandleInfoBoxVersionIdChange);
 		left_vbox.PackStart (info_box, false, true, 0);
-		
+
 		query = new FSpot.PhotoQuery (db.Photos);
 		query.ItemChanged += HandleQueryItemChanged;
 
+#if SHOW_CALENDAR
+		FSpot.SimpleCalendar cal = new FSpot.SimpleCalendar (query);
+		cal.DaySelected += HandleCalendarDaySelected;
+		left_vbox.PackStart (cal, false, true, 0);
+#endif
+
 		group_selector = new FSpot.GroupSelector ();
 		FSpot.GroupAdaptor adaptor = new FSpot.TimeAdaptor (query);
 		//FSpot.GroupAdaptor adaptor = new FSpot.DirectoryAdaptor (query);		
-		//group_selector.Mode = FSpot.GroupSelector.RangeType.Min;
+		group_selector.Mode = FSpot.GroupSelector.RangeType.All;
 
 		group_selector.Adaptor  = adaptor;
 		group_selector.ShowAll ();
 
-		group_vbox.PackStart (group_selector, false, false, 0);
-		group_vbox.ReorderChild (group_selector, 0);
-		
+		view_vbox.PackStart (group_selector, false, false, 0);
+		view_vbox.ReorderChild (group_selector, 0);
+
 		icon_view = new QueryView (query);
 		icon_view_scrolled.Add (icon_view);
 		icon_view.SelectionChanged += new IconView.SelectionChangedHandler (HandleSelectionChanged);
 		icon_view.DoubleClicked += new IconView.DoubleClickedHandler (HandleDoubleClicked);
 		icon_view.GrabFocus ();
 
-		FSpot.PreviewPopup preview = new FSpot.PreviewPopup (icon_view);
+		new FSpot.PreviewPopup (icon_view);
 
 		Gtk.Drag.SourceSet (icon_view, Gdk.ModifierType.Button1Mask | Gdk.ModifierType.Button3Mask,
 				    icon_source_target_table, DragAction.Copy | DragAction.Move);
@@ -230,34 +253,57 @@ public class MainWindow {
 
 		UpdateMenus ();
 		main_window.ShowAll ();
-		
+		main_window.Destroyed += HandleCloseCommand;
+
 		if (Toplevel == null)
 			Toplevel = this;
 	}
 
-	void HandleViewNotebookSwitchPage (object sender, SwitchPageArgs args)
+	// Switching mode.
+	public enum ModeType {
+		IconView,
+		PhotoView
+	};
+
+	ModeType view_mode;
+
+	public ModeType ViewMode {
+		get {
+			return view_mode;
+		}
+	}
+
+	public void SetViewMode (ModeType value)
 	{
-		switch (view_notebook.CurrentPage) {
-		case 0:
-			mode = ModeType.IconView;
+		view_mode = value;
+		switch (view_mode) {
+		case ModeType.IconView:
+			if (view_notebook.CurrentPage != 0)
+				view_notebook.CurrentPage = 0;
+				
 			Present (photo_view.CurrentPhoto);
 			break;
-		case 1:
-			mode = ModeType.PhotoView;
+		case ModeType.PhotoView:
+			if (view_notebook.CurrentPage != 1)
+				view_notebook.CurrentPage = 1;
+			
 			Present (icon_view.FocusCell);
 			break;
 		}
 	}
-
-	void SwitchToIconViewMode ()
+	
+	void HandleViewNotebookSwitchPage (object sender, SwitchPageArgs args)
 	{
-		view_notebook.CurrentPage = 0;
+		switch (view_notebook.CurrentPage) {
+		case 0:
+			SetViewMode (ModeType.IconView);
+			break;
+		case 1:
+			SetViewMode (ModeType.PhotoView);
+			break;
+		}
 	}
 
-	void SwitchToPhotoViewMode ()
-	{
-		view_notebook.CurrentPage = 1;
-	}
 
 	public int [] SelectedIds () {
 		int [] ids;
@@ -265,7 +311,7 @@ public class MainWindow {
 		if (fsview != null)
 			ids = new int [] { fsview.View.CurrentPhoto };
 		else {
-			switch (mode) {
+			switch (view_mode) {
 			case ModeType.IconView:
 				ids = icon_view.SelectedIdxs;
 				break;
@@ -423,9 +469,30 @@ public class MainWindow {
 		args.SelectionData.Set (targets[0], 8, data, data.Length);
 	}
 
-	void HandleAdaptorGlassSet (FSpot.GroupAdaptor sender, int index)
+#if SHOW_CALENDAR
+	void HandleCalendarDaySelected (object sender, System.EventArgs args)
 	{
-		switch (mode) {
+		FSpot.SimpleCalendar cal = sender as FSpot.SimpleCalendar;
+		JumpTo (cal.Date);
+	}
+#endif
+
+	private void JumpTo (System.DateTime time)
+	{
+		//FIXME this should make sure the photos are sorted by
+		//time.  This should be handled via a property that
+		//does all the needed switching.
+		if (!(group_selector.Adaptor is FSpot.TimeAdaptor))
+			HandleArrangeByTime (null, null);
+		
+		FSpot.TimeAdaptor time_adaptor = group_selector.Adaptor as FSpot.TimeAdaptor;
+		if (time_adaptor != null)
+			JumpTo (time_adaptor.LookupItem (time));
+	}
+
+	private void JumpTo (int index)
+	{
+		switch (view_mode) {
 		case ModeType.PhotoView:
 			photo_view.CurrentPhoto = index;
 			break;
@@ -434,7 +501,11 @@ public class MainWindow {
 			icon_view.Throb (index);
 			break;
 		}
+	}
 
+	void HandleAdaptorGlassSet (FSpot.GroupAdaptor sender, int index)
+	{
+		JumpTo (index);
 	}
 
 	//
@@ -446,28 +517,60 @@ public class MainWindow {
 		Photo [] photos = SelectedPhotos ();
 		
 		if (photos.Length > 0) {
-			string thumbnail_path = Thumbnail.PathForUri (photos[0].DefaultVersionUri.ToString (), ThumbnailSize.Large);
-			Pixbuf thumbnail = ThumbnailCache.Default.GetThumbnailForPath (thumbnail_path);
-			if (thumbnail != null) {
-				Gtk.Drag.SetIconPixbuf (args.Context, thumbnail, 0, 0);
-				thumbnail.Dispose ();
+			int len = Math.Min (photos.Length, 4);
+			int size = 48;
+			int border  = 2;
+			int csize = size/2 + len * size / 2 + 2 * border ;
+			
+			Pixbuf container = new Pixbuf (Gdk.Colorspace.Rgb, true, 8, csize, csize);
+			container.Fill (0x00000000);
+
+			bool use_icon = false;;
+			while (len-- > 0) {
+				string thumbnail_path = Thumbnail.PathForUri (photos[len].DefaultVersionUri.ToString (), ThumbnailSize.Large);
+				Pixbuf thumbnail = ThumbnailCache.Default.GetThumbnailForPath (thumbnail_path);
+				if (thumbnail != null) {
+					Pixbuf small = PixbufUtils.ScaleToMaxSize (thumbnail, size, size);				
+
+					int x = border + len * (size/2) + (size - small.Width)/2;
+					int y = border + len * (size/2) + (size - small.Height)/2;
+					Pixbuf box = new Pixbuf (container, x - border, y - border, small.Width + 2 * border, small.Height + 2 * border);
+					box.Fill (0x000000ff);
+
+					small.CopyArea (0, 0, small.Width, small.Height, container, x, y); 
+					
+					thumbnail.Dispose ();
+					small.Dispose ();
+					use_icon = true;
+				}
 			}
+			if (use_icon)
+				Gtk.Drag.SetIconPixbuf (args.Context, container, 0, 0);
+			container.Dispose ();
 		}
 	}
 
 	void HandleIconViewDragDataGet (object sender, DragDataGetArgs args)
 	{		
 		UriList list = new UriList (SelectedPhotos ());
-		Byte [] data = Encoding.UTF8.GetBytes (list.ToString ());
-		Atom [] targets = args.Context.Targets;
+
+		switch (args.Info) {
+		case (uint) TargetType.TagList:
+			Byte [] data = Encoding.UTF8.GetBytes (list.ToString ());
+			Atom [] targets = args.Context.Targets;
 		
-		args.SelectionData.Set (targets[0], 8, data, data.Length);
+			args.SelectionData.Set (targets[0], 8, data, data.Length);
+			break;
+		case (uint) TargetType.RootWindow:
+			System.Console.WriteLine ("Got background drop");
+			HandleSetAsBackgroundCommand (null, null);
+			break;
+		} 
 	}
 
 	void HandleIconViewDragDrop (object sender, DragDropArgs args)
 	{
-		Widget source = Gtk.Drag.GetSourceWidget (args.Context);
-
+		//Widget source = Gtk.Drag.GetSourceWidget (args.Context);
 		//Console.WriteLine ("Drag Drop {0}", source == null ? "null" : source.TypeName);
 		
 		args.RetVal = true;
@@ -475,8 +578,7 @@ public class MainWindow {
 
 	void HandleIconViewDragMotion (object sender, DragMotionArgs args)
 	{
-		Widget source = Gtk.Drag.GetSourceWidget (args.Context);
-
+		//Widget source = Gtk.Drag.GetSourceWidget (args.Context);
 		//Console.WriteLine ("Drag Motion {0}", source == null ? "null" : source.TypeName);
 
 		Gdk.Drag.Status (args.Context, args.Context.SuggestedAction, args.Time);
@@ -534,13 +636,16 @@ public class MainWindow {
 
 	void HandleSelectionChanged (IconView view)
 	{
+		info_box.Photo = CurrentPhoto;
+		if (info_display != null)
+			info_display.Photo = CurrentPhoto;
 		UpdateMenus ();
 	}
 
 	void HandleDoubleClicked (IconView icon_view, int clicked_item)
 	{
 		icon_view.FocusCell = clicked_item;
-		SwitchToPhotoViewMode ();
+		SetViewMode (ModeType.PhotoView);
 	}
 
 	//
@@ -572,7 +677,7 @@ public class MainWindow {
 	void HandlePhotoViewButtonPressEvent (object sender, Gtk.ButtonPressEventArgs args)
 	{
 		if (args.Event.Type == EventType.TwoButtonPress && args.Event.Button == 1)
-			SwitchToIconViewMode ();
+			SetViewMode (ModeType.IconView);
 	}
 
 	void HandlePhotoViewUpdateStarted (PhotoView sender)
@@ -595,8 +700,7 @@ public class MainWindow {
 
 	void HandlePhotoViewDragDrop (object sender, DragDropArgs args)
 	{
-		Widget source = Gtk.Drag.GetSourceWidget (args.Context);
-		
+		//Widget source = Gtk.Drag.GetSourceWidget (args.Context);
 		//Console.WriteLine ("Drag Drop {0}", source == null ? "null" : source.TypeName);
 
 		args.RetVal = true;
@@ -604,8 +708,7 @@ public class MainWindow {
 
 	void HandlePhotoViewDragMotion (object sender, DragMotionArgs args)
 	{
-		Widget source = Gtk.Drag.GetSourceWidget (args.Context);
-
+		//Widget source = Gtk.Drag.GetSourceWidget (args.Context);
 		//Console.WriteLine ("Drag Motion {0}", source == null ? "null" : source.TypeName);
 
 		Gdk.Drag.Status (args.Context, args.Context.SuggestedAction, args.Time);
@@ -614,8 +717,7 @@ public class MainWindow {
 
 	void HandlePhotoViewDragDataReceived (object sender, DragDataReceivedArgs args)
 	{
-	 	Widget source = Gtk.Drag.GetSourceWidget (args.Context);     
-		
+	 	//Widget source = Gtk.Drag.GetSourceWidget (args.Context);     
 		//Console.WriteLine ("Drag received {0}", source == null ? "null" : source.TypeName);
 
 		HandleAttachTagCommand (sender, null);
@@ -669,79 +771,10 @@ public class MainWindow {
 		}
 	}
 
-#if false
 	unsafe void HandlePrintCommand (object sender, EventArgs e)
 	{
-		PrintJob pj = new PrintJob (PrintConfig.Default ());
-		PrintDialog dialog = new PrintDialog (pj, "Print Images", 0);
-
-		PrintContext ctx = pj.Context;
-		double page_width, page_height;
-		pj.GetPageSize (out page_width, out page_height);
-
-		foreach (Photo photo in SelectedPhotos ()) {
-			Print.Beginpage (ctx, "F-Spot "+ photo.DefaultVersionPath);
-			
-			Pixbuf image = FSpot.PhotoLoader.Load (photo);
-
-			bool rotate = false;
-			double width = page_width;
-			double height = page_height;
-			if (image.Width > image.Height) {
-				rotate = true;
-				width = page_height;
-				height = page_width;
-			}
-			
-			double scale = Math.Min (width / image.Width, 
-						  height / image.Height);
-			
-			Gnome.Print.Gsave (ctx);
-
-			if (rotate) {
-				Gnome.Print.Rotate (ctx, 90);
-				Gnome.Print.Translate (ctx, 0, -page_width);
-			}
-			Print.Translate (ctx, 
-					 (width - image.Width * scale) / 2.0, 
-					 (height - image.Height * scale) / 2.0);
-
-			Print.Scale (ctx, image.Width * scale, image.Height * scale);
-
-			Print.Pixbuf (ctx, image);
-			Print.Grestore (ctx);
-
-			//Print.Show (ctx, photo.Description);
-			Print.Showpage (ctx);
-			image.Dispose ();
-		}
-
-		int response = dialog.Run ();
-
-		Console.WriteLine ("response: " + response);
-
-		if (response == (int) PrintButtons.Cancel) {
-			dialog.Destroy ();
-		}
-		pj.Close ();
-
-		switch (response) {
-		case (int) PrintButtons.Print:
-			pj.Print ();
-			break;
-		case (int) PrintButtons.Preview:
-			new PrintJobPreview (pj, "Testing").Show ();
-			break;
-		}
-
-		dialog.Destroy ();
+		new FSpot.PrintDialog (SelectedPhotos ());
 	}
-#else
-	unsafe void HandlePrintCommand (object sender, EventArgs e)
-	{
-		FSpot.PrintDialog dialog = new FSpot.PrintDialog (SelectedPhotos ());
-	}
-#endif	
 
 	private Gtk.Dialog info_display_window;
 	public void HandleInfoDisplayDestroy (object sender, EventArgs args)
@@ -774,23 +807,42 @@ public class MainWindow {
 
 	void HandleExportToGallery (object sender, EventArgs args)
 	{
-		FSpot.GalleryExport export = new FSpot.GalleryExport (new FSpot.PhotoArray (SelectedPhotos ()));
+		new FSpot.GalleryExport (new FSpot.PhotoArray (SelectedPhotos ()));
 	}
 
 	void HandleExportToVfs (object sender, EventArgs args)
 	{
-		FSpot.VfsExport export = new FSpot.VfsExport (new FSpot.PhotoArray (SelectedPhotos ()));
+		new FSpot.VfsExport (new FSpot.PhotoArray (SelectedPhotos ()));
+	}
+
+	void HandleViewDirectory (object sender, EventArgs args)
+	{
+		Gtk.Window win = new Gtk.Window ("Directory View");
+		IconView view = new IconView (new FSpot.DirectoryCollection (System.IO.Directory.GetCurrentDirectory ()));
+		new FSpot.PreviewPopup (view);
+
+		view.DisplayTags = false;
+
+		Gtk.ScrolledWindow scrolled = new ScrolledWindow ();
+		win.Add (scrolled);
+		scrolled.Add (view);
+		win.ShowAll ();
 	}
 
 	void HandleExportToFlickr (object sender, EventArgs args)
 	{
-		FSpot.FlickrExport export = new FSpot.FlickrExport (new FSpot.PhotoArray (SelectedPhotos ()));
+		new FSpot.FlickrExport (new FSpot.PhotoArray (SelectedPhotos ()));
 	}
 	
 	void HandleExportToFotki (object sender, EventArgs args)
 	{
 		
 	}
+	
+	void HandleExportToCD (object sender, EventArgs args)
+	{
+		new FSpot.CDExport (new FSpot.PhotoArray (SelectedPhotos ()));
+	}
 
 	void HandleSendMailCommand (object sender, EventArgs args)
 	{
@@ -833,7 +885,9 @@ public class MainWindow {
                 // * E.g. "Martin Willemoes Hansen"
                 string translators = Mono.Posix.Catalog.GetString ("translator-credits");
 
-                new About ("F-Spot", "0.0.3", "Copyright 2003-2004 Novell Inc.",
+                new About (FSpot.Defines.PACKAGE, 
+			   FSpot.Defines.VERSION, 
+			   "Copyright 2003-2005 Novell Inc.",
                            null, authors, null, translators, null).Show();
 	}
 
@@ -908,21 +962,17 @@ public class MainWindow {
 	public void HandleCreateNewTagCommand (object sender, EventArgs args)
 	{
 		TagCommands.Create command = new TagCommands.Create (db.Tags, main_window);
-		if (command.Execute (TagCommands.TagType.Tag))
-			tag_selection_widget.Update ();
+		command.Execute (TagCommands.TagType.Tag);
 	}
 
 	public void HandleCreateNewCategoryCommand (object sender, EventArgs args)
 	{
 		TagCommands.Create command = new TagCommands.Create (db.Tags, main_window);
-		if (command.Execute (TagCommands.TagType.Category))
-			tag_selection_widget.Update ();
+		command.Execute (TagCommands.TagType.Category);
 	}
 
 	public void HandleAttachTagCommand (object obj, EventArgs args)
 	{
-		Tag [] tags = this.tag_selection_widget.TagHighlight ();
-		
 		AttachTags (tag_selection_widget.TagHighlight (), SelectedIds ());
 	}
 
@@ -950,14 +1000,14 @@ public class MainWindow {
 			return;
 		
 		TagCommands.Edit command = new TagCommands.Edit (db, main_window);
-		if (command.Execute (tags [0]))
-			tag_selection_widget.Update ();
+		command.Execute (tags [0]);
 	}
 
 	void HandleAdjustColor (object sender, EventArgs args)
 	{
 		if (ActiveIndex () > 0) {
-			new FSpot.ColorDialog (query, ActiveIndex ());
+			SetViewMode (ModeType.PhotoView);
+			new FSpot.ColorDialog (photo_view.View);
 		}
 	}
 
@@ -1048,15 +1098,18 @@ public class MainWindow {
 		icon_view.DisplayDates = !icon_view.DisplayDates;
 	}
 
+	void HandleDisplayGroupSelector (object sender, EventArgs args)
+	{
+		if (group_selector.Visible)
+			group_selector.Hide ();
+		else
+			group_selector.Show ();
+	}
+
 	void HandleViewSlideShow (object sender, EventArgs args)
 	{
-#if true
 		main_window.GdkWindow.Cursor = new Gdk.Cursor (Gdk.CursorType.Watch);
-                GLib.Idle.Add (new GLib.IdleHandler (SlideShow));
-#else
-		SlideCommands.Create command = new SlideCommands.Create (query.Photos);
-		command.Execute ();
-#endif
+		slide_delay.Start ();
 	}
 
 	private bool SlideShow ()
@@ -1124,6 +1177,16 @@ public class MainWindow {
 		args.RetVal = true;
 	}
 
+	void HandleViewBrowse (object sender, EventArgs args)
+	{
+		SetViewMode (ModeType.IconView);
+	}
+
+	void HandleViewPhoto (object sender, EventArgs args)
+	{
+		SetViewMode (ModeType.PhotoView);
+	}
+
 	void HandleViewFullscreen (object sender, EventArgs args)
 	{
 		int active = Math.Max (ActiveIndex (), 0);
@@ -1139,7 +1202,7 @@ public class MainWindow {
 	
 	void Present (int item)
 	{
-		switch (mode) {
+		switch (view_mode) {
 		case ModeType.IconView:
 			icon_view.ScrollTo (item);
 			icon_view.Throb (item);
@@ -1158,13 +1221,13 @@ public class MainWindow {
 	
 	void HandleZoomOut (object sender, EventArgs args)
 	{
-		switch (mode) {
+		switch (view_mode) {
 		case ModeType.PhotoView:
 			double old_zoom = photo_view.Zoom;
 
 			old_zoom /= FSpot.PhotoImageView.ZoomMultipler;
 			if (old_zoom < .001) {
-				SwitchToIconViewMode ();
+				SetViewMode (ModeType.IconView);
 			} else {
 				photo_view.Zoom = old_zoom;
 			}
@@ -1183,7 +1246,7 @@ public class MainWindow {
 
 	void HandleZoomIn (object sender, EventArgs args)
 	{
-		switch (mode) {
+		switch (view_mode) {
 		case ModeType.PhotoView:
 			double old_zoom = photo_view.Zoom;
 			try {
@@ -1200,7 +1263,7 @@ public class MainWindow {
 			width = Math.Max (width, 64);
 			if (width >= 512) {
 				photo_view.Zoom = 0.0;
-				SwitchToPhotoViewMode ();
+				SetViewMode (ModeType.PhotoView);
 			} else {
 				icon_view.ThumbnailWidth = width;
 			}			
@@ -1246,7 +1309,6 @@ public class MainWindow {
 		Tag [] tags = this.tag_selection_widget.TagHighlight ();
 		
 		db.Photos.Remove (tags);
-		tag_selection_widget.Update ();
 		icon_view.QueueDraw ();
 	}
 
@@ -1263,13 +1325,11 @@ public class MainWindow {
 
 	public void HandleRotate90Command (object sender, EventArgs args)
 	{
-		Console.WriteLine ("Rotate Left");
 		RotateSelectedPictures (RotateCommand.Direction.Clockwise);
 	}
 
 	public void HandleRotate270Command (object sender, EventArgs args)
 	{
-		Console.WriteLine ("Rotate Left");
 		RotateSelectedPictures (RotateCommand.Direction.Counterclockwise);
 	}
 
@@ -1354,7 +1414,7 @@ public class MainWindow {
 
 	void OnTagSelectionChanged (object obj)
 	{
-		SwitchToIconViewMode ();
+		SetViewMode (ModeType.IconView);
 		UpdateQuery ();
 	}
 	
@@ -1363,10 +1423,9 @@ public class MainWindow {
 		UpdateMenus ();
 	}
 
-	void HandleQueryItemChanged (FSpot.IPhotoCollection query, int item)
+	void HandleQueryItemChanged (FSpot.IBrowsableCollection browsable, int item)
 	{
-		Photo photo = query.Photos [item];
-		if (info_box.Photo == photo)
+		if (info_box.Photo == (browsable.Items[item] as Photo))
 			info_box.Update ();
 	}
 	//
