@@ -71,17 +71,17 @@ namespace FSpot {
 				return;
 			
 			int idx = CurrentPhoto;
-			CurrentPhoto = 0;
+			CurrentPhoto = -1;
 			CurrentPhoto = idx;
 		}
 
-		private void HandleQueryChanged (IPhotoCollection query)
+		private void HandleQueryChanged (IBrowsableCollection browsable)
 		{
-			if (query == this.query)
+			if (query == browsable)
 				Reload ();
 		}
 
-		public void HandleQueryItemChanged (IPhotoCollection query, int item)
+		public void HandleQueryItemChanged (IBrowsableCollection browsable, int item)
 		{
 			if (item == CurrentPhoto)
 				Reload ();
@@ -111,12 +111,14 @@ namespace FSpot {
 		{
 			Gdk.Pixbuf prev = this.Pixbuf;
 			Gdk.Pixbuf next = loader.Pixbuf;
-			
+
+#if SPEED_COPY_DATA
 			if (next != null && prev != null && next.Width == prev.Width && prev.Height == next.Height)
 				prev.CopyArea (0, 0, next.Width, next.Height, next, 0, 0);
 			else
 				next.Fill (0x00000000);
-			
+#endif
+
 			this.Pixbuf = next;
 			if (prev != null)
 				prev.Dispose ();
@@ -166,8 +168,22 @@ namespace FSpot {
 				return;
 
 			if (load_async) {
+				try {
 				loader.Load (Photo.DefaultVersionPath);
-			} else {
+				} catch (System.Exception e) {
+					// FIXME we should check the exception type and do something
+					// like offer the user a chance to locate the moved file and
+					// update the db entry, but for now just set the error pixbuf
+
+					Gdk.Pixbuf old = this.Pixbuf;
+					this.Pixbuf = new Gdk.Pixbuf (PixbufUtils.ErrorPixbuf, 0, 0, 
+								      PixbufUtils.ErrorPixbuf.Width, 
+								      PixbufUtils.ErrorPixbuf.Height);
+					if (old != null)
+						old.Dispose ();
+				}
+
+			} else {	
 				Gdk.Pixbuf old = this.Pixbuf;
 				this.Pixbuf = FSpot.PhotoLoader.Load (Query, current_photo);
 				if (old != null)
@@ -290,6 +306,7 @@ namespace FSpot {
 		{
 			loader.AreaUpdated -= HandlePixbufAreaUpdated;
 			loader.AreaPrepared -= HandlePixbufPrepared;
+			loader.Dispose ();
 		}
 
 		protected override bool OnDestroyEvent (Gdk.Event evnt)
