@@ -8,7 +8,7 @@ using System.Text;
 using System;
 
 
-public class Photo : DbItem, IComparable {
+public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 	// IComparable 
 	public int CompareTo (object obj) {
 		if (this.GetType () == obj.GetType ()) {
@@ -96,10 +96,13 @@ public class Photo : DbItem, IComparable {
 		}
 	}
 
-	private ArrayList tags = new ArrayList ();
+	private ArrayList tags;
 	public Tag [] Tags {
 		get {
-			return (Tag []) tags.ToArray (typeof (Tag));
+			if (tags == null)
+				return new Tag [0];
+			else 
+				return (Tag []) tags.ToArray (typeof (Tag));
 		}
 	}
 
@@ -127,12 +130,14 @@ public class Photo : DbItem, IComparable {
 	public const int OriginalVersionId = 1;
 	private uint highest_version_id;
 
-	private Hashtable version_names = new Hashtable ();
+	private Hashtable version_names;
 
 	public uint [] VersionIds {
 		get {
-			uint [] ids = new uint [version_names.Count];
+			if (version_names == null)
+				return new uint [0];
 
+			uint [] ids = new uint [version_names.Count];
 			uint i = 0;
 			foreach (uint id in version_names.Keys)
 				ids [i ++] = id;
@@ -157,6 +162,9 @@ public class Photo : DbItem, IComparable {
 	// it's supposed to be used only within the Photo and PhotoStore classes.
 	public void AddVersionUnsafely (uint version_id, string name)
 	{
+		if (version_names == null)
+			version_names = new Hashtable ();
+
 		version_names [version_id] = name;
 
 		highest_version_id = Math.Max (version_id, highest_version_id);
@@ -171,10 +179,13 @@ public class Photo : DbItem, IComparable {
 					       + " (" + version_name + ")" + extension);
 	}
 
-	public bool VersionNameExists (string version_name)	{
-		foreach (string n in version_names.Values) {
-			if (n == version_name)
-				return true;
+	public bool VersionNameExists (string version_name)
+	{
+		if (version_names != null) {
+			foreach (string n in version_names.Values) {
+				if (n == version_name)
+					return true;
+			}
 		}
 
 		return false;
@@ -182,7 +193,10 @@ public class Photo : DbItem, IComparable {
 
 	public string GetVersionName (uint version_id)
 	{
-		return version_names [version_id] as string;
+		if (version_names != null)
+			return version_names [version_id] as string;
+		else 
+			return null;
 	}
 
 	public string GetVersionPath (uint version_id)
@@ -247,6 +261,9 @@ public class Photo : DbItem, IComparable {
 			PhotoStore.GenerateThumbnail (new_path);
 		}
 
+		if (version_names == null)
+			version_names = new Hashtable ();
+
 		highest_version_id ++;
 		version_names [highest_version_id] = name;
 
@@ -299,6 +316,9 @@ public class Photo : DbItem, IComparable {
 	// This doesn't check if the tag is already there, use with caution.
 	public void AddTagUnsafely (Tag tag)
 	{
+		if (tags == null)
+			tags = new ArrayList ();
+
 		tags.Add (tag);
 	}
 
@@ -384,8 +404,6 @@ public class PhotoStore : DbStore {
 	//
 	static Pixbuf GenerateFromExif (string path, string uri)
 	{
-		Pixbuf pixbuf;
-
 		try {
 			using (ExifData ed = new ExifData (path)){
 				byte [] thumbData = ed.Data;
@@ -413,16 +431,16 @@ public class PhotoStore : DbStore {
 		string uri = UriList.PathToFileUri (path).ToString ();
 		Pixbuf thumbnail = null;
 
-		if (use_exif)
+		if (use_exif) {
 			thumbnail = GenerateFromExif (path, uri);
-
-		if (thumbnail == null)
-			thumbnail = PhotoStore.ThumbnailFactory.GenerateThumbnail (uri, "image/jpeg");
-		
-		// FIXME if this is null then the file doesn't exist.
-		if (thumbnail != null) {
-			PhotoStore.ThumbnailFactory.SaveThumbnail (thumbnail, uri, File.GetLastWriteTime (path));
 		}
+
+		// Save EXIF generated thumbnails in a silightly invalid way so that we know to regnerate them.
+		if (thumbnail != null) 
+			thumbnail.Save (Thumbnail.PathForUri (uri, ThumbnailSize.Large), "png");
+		else 
+			thumbnail = PixbufUtils.GenerateThumbnail (path);
+			
 		
 		return thumbnail;
 	}
@@ -971,6 +989,77 @@ public class PhotoStore : DbStore {
 		return Query (tags, null);
 	}
 
+	public Photo [] Query (string query)
+	{
+		Console.WriteLine ("Query Start {0}", System.DateTime.Now.ToLongTimeString ());
+
+		SqliteCommand command = new SqliteCommand ();
+		command.Connection = Connection;
+		command.CommandText = query;
+		SqliteDataReader reader = command.ExecuteReader ();
+		
+		Console.WriteLine ("Query Mid {0}", System.DateTime.Now);
+
+		ArrayList version_list = new ArrayList ();
+		ArrayList id_list = new ArrayList ();
+		while (reader.Read ()) {
+			uint id = Convert.ToUInt32 (reader [0]);
+			Photo photo = LookupInCache (id) as Photo;
+
+			if (photo == null) {
+				photo = new Photo (id,
+						   Convert.ToUInt32 (reader [1]),
+						   reader [2].ToString (),
+						   reader [3].ToString ());
+				
+				photo.Description = reader[4].ToString ();
+				photo.DefaultVersionId = Convert.ToUInt32 (reader[5]);		 
+				
+				version_list.Add (photo);
+			}
+
+			id_list.Add (photo);
+		}
+
+		Console.WriteLine ("Query End {0}", System.DateTime.Now.ToLongTimeString ());
+
+		bool need_load = false;
+		Console.WriteLine ("Start {0}", System.DateTime.Now);
+		foreach (Photo photo in version_list) {
+			AddToCache (photo);
+			need_load |= !photo.Loaded;
+		}
+		
+		if (need_load) {
+			GetAllTags ();
+			GetAllVersions ();
+			foreach (Photo photo in version_list) {
+				photo.Loaded = true;
+			}
+		} else {
+			//Console.WriteLine ("Skipped Loading Data");
+		}
+
+		Console.WriteLine ("End {0}", System.DateTime.Now);
+		command.Dispose ();
+
+		return id_list.ToArray (typeof (Photo)) as Photo [];
+	}
+
+	public Photo [] Query (System.IO.DirectoryInfo dir)
+	{
+		string query_string = String.Format ("SELECT photos.id,                          " +
+						     "       photos.time,                        " +
+						     "       photos.directory_path,              " +
+						     "       photos.name,                        " +
+						     "       photos.description,                 " +
+						     "       photos.default_version_id           " +
+						     "     FROM photos                           " +
+						     "     WHERE directory_path = \"{0}\"", dir.FullName);
+
+		return Query (query_string);
+	}	   
+
 	public Photo [] Query (Tag [] tags, DateRange range)
 	{
 		string query;
@@ -1041,61 +1130,7 @@ public class PhotoStore : DbStore {
 		query_builder.Append ("ORDER BY photos.time");
 		query = query_builder.ToString ();
 		
-		Console.WriteLine ("Query Start {0}", System.DateTime.Now.ToLongTimeString ());
-
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-		command.CommandText = query;
-		SqliteDataReader reader = command.ExecuteReader ();
-		
-		Console.WriteLine ("Query Mid {0}", System.DateTime.Now);
-
-		ArrayList version_list = new ArrayList ();
-		ArrayList id_list = new ArrayList ();
-		while (reader.Read ()) {
-			uint id = Convert.ToUInt32 (reader [0]);
-			Photo photo = LookupInCache (id) as Photo;
-
-			if (photo == null) {
-				photo = new Photo (id,
-						   Convert.ToUInt32 (reader [1]),
-						   reader [2].ToString (),
-						   reader [3].ToString ());
-				
-				photo.Description = reader[4].ToString ();
-				photo.DefaultVersionId = Convert.ToUInt32 (reader[5]);		 
-				
-				version_list.Add (photo);
-			}
-
-			id_list.Add (photo);
-		}
-
-		Console.WriteLine ("Query End {0}", System.DateTime.Now.ToLongTimeString ());
-
-		bool need_load = false;
-		Console.WriteLine ("Start {0}", System.DateTime.Now);
-		foreach (Photo photo in version_list) {
-			AddToCache (photo);
-			need_load |= !photo.Loaded;
-		}
-		
-		if (need_load) {
-			GetAllTags ();
-			GetAllVersions ();
-			foreach (Photo photo in version_list) {
-				photo.Loaded = true;
-			}
-		} else {
-			//Console.WriteLine ("Skipped Loading Data");
-		}
-
-
-		Console.WriteLine ("End {0}", System.DateTime.Now);
-
-		command.Dispose ();
-
-		return id_list.ToArray (typeof (Photo)) as Photo [];
+		return Query (query);
 	}
 
 #if TEST_PHOTO_STORE
